

import mock
import unittest2

from thanatos import ccp_image_server


class CCPImageServerTestCase(unittest2.TestCase):

    def setUp(self):
        pass

    @mock.patch('thanatos.ccp_image_server.get_image_server_link')
    def test_get_character_image_links(self, mock_get_image_server_link):
        """ Test the character image links makes all the right calls to get_image_server_link. """

        links = ccp_image_server.get_character_image_links(123456)

        mock_get_image_server_link.assert_any_call(123456, 'char', 32)
        mock_get_image_server_link.assert_any_call(123456, 'char', 64)
        mock_get_image_server_link.assert_any_call(123456, 'char', 128)
        mock_get_image_server_link.assert_any_call(123456, 'char', 256)
        mock_get_image_server_link.assert_any_call(123456, 'char', 512)
        mock_get_image_server_link.assert_any_call(123456, 'char', 1024)
        self.assertEqual(mock_get_image_server_link.call_count, 6)

    @mock.patch('thanatos.ccp_image_server.get_image_server_link')
    def test_get_corporation_image_links(self, mock_get_image_server_link):
        """ Test the corporation image links makes all the right calls to get_image_server_link. """

        links = ccp_image_server.get_corporation_image_links(123456)

        mock_get_image_server_link.assert_any_call(123456, 'corp', 32)
        mock_get_image_server_link.assert_any_call(123456, 'corp', 64)
        mock_get_image_server_link.assert_any_call(123456, 'corp', 128)
        mock_get_image_server_link.assert_any_call(123456, 'corp', 256)
        self.assertEqual(mock_get_image_server_link.call_count, 4)

    @mock.patch('thanatos.ccp_image_server.get_image_server_link')
    def test_get_type_image_links(self, mock_get_image_server_link):
        """ Test the type image links makes all the right calls to get_image_server_link. """

        links = ccp_image_server.get_type_links(123456)

        mock_get_image_server_link.assert_any_call(123456, 'type', 32)
        mock_get_image_server_link.assert_any_call(123456, 'type', 64)
        mock_get_image_server_link.assert_any_call(123456, 'type', 128)
        mock_get_image_server_link.assert_any_call(123456, 'type', 256)
        mock_get_image_server_link.assert_any_call(123456, 'type', 512)
        self.assertEqual(mock_get_image_server_link.call_count, 5)

    def test_get_image_server_link_char(self):
        """ Test get_image_link returns the correct URL for a character. """

        image_link = ccp_image_server.get_image_server_link(123456, 'char', 32)

        self.assertEqual(image_link, 'https://image.eveonline.com/Character/123456_32.jpg')

    def test_get_image_server_link_corp(self):
        """ Test get_image_link returns the correct URL for a corporation. """

        image_link = ccp_image_server.get_image_server_link(123456, 'corp', 32)

        self.assertEqual(image_link, 'https://image.eveonline.com/Corporation/123456_32.png')

    def test_get_image_server_link_alliance(self):
        """ Test get_image_link returns the correct URL for a alliance. """

        image_link = ccp_image_server.get_image_server_link(123456, 'alli', 32)

        self.assertEqual(image_link, 'https://image.eveonline.com/Alliance/123456_32.png')

    def test_get_image_server_link_faction(self):
        """ Test get_image_link returns the correct URL for a faction. """

        image_link = ccp_image_server.get_image_server_link(123456, 'fac', 32)

        self.assertEqual(image_link, 'https://image.eveonline.com/Alliance/123456_32.png')

    def test_get_image_server_link_other(self):
        """ Test get_image_link returns the correct URL for other. """

        image_link = ccp_image_server.get_image_server_link(123456, 'other', 32)

        self.assertEqual(image_link, 'https://image.eveonline.com/Render/123456_32.png')




def get_character_image_links(character_id):
    """ Takes a character ID and returns a dict with all available size links.

    :param character_id: A character ID from EVE.

    :return: A dict of size and urls.
    """

    sizes = 32, 64, 128, 256, 512, 1024
    return_dict = {x: get_image_server_link(character_id, 'char', x) for x in sizes}

    return return_dict


def get_corporation_image_links(corporation_id):
    """ Takes a character ID and returns a dict with all available size links.

    :param corporation_id: A corporation ID from EVE.

    :return: A dict of size and urls.
    """

    sizes = 32, 64, 128, 256
    return_dict = {x: get_image_server_link(corporation_id, 'corp', x) for x in sizes}

    return return_dict


def get_type_links(type_id):
    """ Takes a type ID and returns a dict with all available size links.

    :param corporation_id: An EVE typeID.

    :return: A dict of size and urls.
    """

    sizes = 32, 64, 128, 256, 512
    return_dict = {x: get_image_server_link(type_id, 'type', x) for x in sizes}

    return return_dict


def get_image_server_link(image_id, image_type, icon_size=128):
    """Takes an ID and type name and builds and image server URL.
    Known valid sizes:
    Character: 32, 64, 128, 256, 512, 1024
    Corporation: 32, 64, 128, 256
    Alliance: 32, 64, 128
    Faction: 32, 64, 128
    Types: 32, 64, 128, 256, 512

    :param image_id: ID to be used in the URL.
    :param image_type: Valid string options are: char, corp, alli, fac, type
    :param icon_size: See doc string for valid image sizes.

    :return: A full URL to the CCP image server.
    """

    base_image_server_url = 'https://image.eveonline.com/'

    if image_type == 'char':
        url_type   = 'Character'
        url_format = 'jpg'

    elif image_type == 'corp':
        url_type   = 'Corporation'
        url_format = 'png'

    elif image_type == 'alli':
        url_type   = 'Alliance'
        url_format = 'png'

    elif image_type == 'fac':
        url_type   = 'Alliance'
        url_format = 'png'

    else:
        url_type   = 'Render'
        url_format = 'png'

    full_url = "%s%s/%s_%s.%s" % (base_image_server_url, url_type, image_id, icon_size, url_format)

    return full_url



import logging

from thanatos import database
from thanatos import questions
from thanatos import categories
from thanatos import ccp_image_server

__version__ = '0.1.0'

_log = logging.getLogger('thanatos')





import mock
import unittest2

from thanatos.database import db_utils


class DatabaseUtilsTestCase(unittest2.TestCase):

    def setUp(self):
        pass

    @mock.patch('os.environ')
    def test_db_connection_gets_c9_default(self, mock_environ):
        """ Test that the default connection details get set correctly when run on C9. """

        mock_environ.get.side_effect = [
            'thanatos',
            '0.0.0.0',
            'cloud 9',
        ]

        db_connection_details = db_utils.get_default_connection_details()

        self.assertEqual(db_connection_details, {
            'host': '0.0.0.0',
            'user': 'cloud 9',
            'password': '',
            'database': 'c9',
        })

    @mock.patch('os.environ')
    def test_db_connection_defaults(self, mock_environ):
        """ Test that the default connection details get set correctly when not run on C9. """

        mock_environ.get.return_value = None

        db_connection_details = db_utils.get_default_connection_details()

        self.assertEqual(db_connection_details, {
            'host': '127.0.0.1',
            'user': 'vagrant',
            'password': 'vagrant',
            'database': 'thanatos',
        })

    @mock.patch('urllib.urlretrieve')
    def test_download_tables(self, mock_urlretrieve):
        base_url    = 'http://example.com/'
        tables_list = [
            'table_one',
            'table_two',
        ]

        db_utils.download_tables(tables_list, base_url)

        mock_urlretrieve.assert_any_call('http://example.com/table_one.sql.bz2', 'table_one.sql.bz2')
        mock_urlretrieve.assert_any_call('http://example.com/table_two.sql.bz2', 'table_two.sql.bz2')




import random
import logging

from abc import ABCMeta, abstractmethod

from thanatos.ccp_image_server import get_type_links

_log = logging.getLogger('thanatos.questions.base')


class Question(object):
    """ The base class that all questions are subclasses of. """

    __metaclass__ = ABCMeta

    name = None
    question = None
    description = None
    random_weight = 0
    category = None
    sub_category = None

    def __init__(self, db_connection):
        self.db_connection = db_connection

    @abstractmethod
    def ask(self):
        """ ask is called whenever attempting to get a question.

        :return: A dictionary of answer, question, and other details.
        :rtype: dict
        """

        pass

    def format_question(self, correct_answer, possible_wrong_answers, question, add_images_to_question=False):
        """ Takes a set of values and converts it to the standard format
        expected as a return from self.ask().

        :param correct_answer: A tuple of the answer key and text to be shown.
        :type correct_answer: tuple

        :param possible_wrong_answers: A list of tuples, each tuple being being
        (id, display_name).
        :type possible_wrong_answers: list

        :param question: A string to be set as the question that is asked.
        :type question: string

        :return: A dict that matches what is expected as the return value for self.ask()
        :rtype: dict
        """

        # Lets randomly select some wrong answers
        wrong_answers = random.sample(possible_wrong_answers, 2)

        # Combine the selected wrong answers with the actual answer and shuffle
        choices = [x for x in wrong_answers]
        choices.append(correct_answer)
        random.shuffle(choices)

        # Format it all in a nice dict
        question = {
            'answer': correct_answer[0],
            'question': question,
            'choices': self.convert_choices_to_dict(choices)
        }

        if add_images_to_question is True:
            question['images'] = get_type_links(correct_answer[0])

        return question

    @staticmethod
    def convert_choices_to_dict(choices):
        """ Takes a list of tuples and converts it to a list of dictionaries of
        where each dictionary has a value and text key. This is the expected format
        of question choices to be returned by self.ask()

        :param convert_choices_to_dict:
        :type convert_choices_to_dict: list

        :return:
        :rtype:
        """

        formatted_choices = []

        for x in choices:
            formatted_choices.append({
                'value': x[0],
                'text': x[1],
            })

        return formatted_choices





import mock
import MySQLdb
import unittest2

from thanatos.questions.base import Question


class QuestionsBaseTestCase(unittest2.TestCase):

    def setUp(self):
        self.mock_db_connection = mock.Mock(spec=MySQLdb.connection)

    def test_class_initializes(self):
        """ Simply test we can create an instance of the Test Question class. """

        TestableBaseQuestion(self.mock_db_connection)
    
    def test_basic_ask(self):
        """ If this somehow breaks... something great has gone wrong. This test
        simple exists to help bump the test coverage number up. The ask method
        shouldn't actually do anything."""

        TestableBaseQuestion(self.mock_db_connection).ask()

    @mock.patch('thanatos.questions.base.Question')
    @mock.patch('random.sample')
    @mock.patch('random.shuffle')
    def test_format_question(self, mock_shuffle, mock_sample, mock_question):
        """ Test that the format_question method returns the correct data. """

        question = 'What region is the best test region?'
        correct_answer = (10000001L, 'Derelik')
        possible_wrong_answers = [
            (10000004L, 'UUA-F4'),
            (10000017L, 'J7HZ-F'),
        ]

        mock_sample.return_value = possible_wrong_answers

        test_question = TestableBaseQuestion(self.mock_db_connection)
        question = test_question.format_question(correct_answer, possible_wrong_answers, question)

        mock_shuffle.assert_called_with([
            (10000004L, 'UUA-F4'),
            (10000017L, 'J7HZ-F'),
            (10000001L, 'Derelik'),
        ])

        mock_question.convert_choices_to_dict.assert_was_called()
        mock_question.convert_choices_to_dict.return_value = [
            {'text': 'UUA-F4',  'value': 10000004L},
            {'text': 'J7HZ-F',  'value': 10000017L},
            {'text': 'Derelik', 'value': 10000001L},
        ]

        self.assertEqual(question, {
            'answer'   : 10000001L,
            'question' : 'What region is the best test region?',
            'choices'  : [
                {'text': 'UUA-F4',  'value': 10000004L},
                {'text': 'J7HZ-F',  'value': 10000017L},
                {'text': 'Derelik', 'value': 10000001L},
            ],
        })

    @mock.patch('thanatos.questions.base.Question')
    @mock.patch('random.sample')
    @mock.patch('random.shuffle')
    def test_format_question_with_image(self, mock_shuffle, mock_sample, mock_question):
        """ Test that the format_question method returns the correct data specifically also with the image link. """

        question = 'What region is the best test region?'
        correct_answer = (10000001L, 'Derelik')
        possible_wrong_answers = [
            (10000004L, 'UUA-F4'),
            (10000017L, 'J7HZ-F'),
        ]

        mock_sample.return_value = possible_wrong_answers

        test_question = TestableBaseQuestion(self.mock_db_connection)
        question = test_question.format_question(correct_answer, possible_wrong_answers, question, add_images_to_question=True)

        mock_shuffle.assert_called_with([
            (10000004L, 'UUA-F4'),
            (10000017L, 'J7HZ-F'),
            (10000001L, 'Derelik'),
        ])

        mock_question.convert_choices_to_dict.assert_was_called()
        mock_question.convert_choices_to_dict.return_value = [
            {'text': 'UUA-F4',  'value': 10000004L},
            {'text': 'J7HZ-F',  'value': 10000017L},
            {'text': 'Derelik', 'value': 10000001L},
        ]

        self.assertEqual(question, {
            'answer'   : 10000001L,
            'question' : 'What region is the best test region?',
            'choices'  : [
                {'text': 'UUA-F4',  'value': 10000004L},
                {'text': 'J7HZ-F',  'value': 10000017L},
                {'text': 'Derelik', 'value': 10000001L},
            ],
            'images': {
                32: 'https://image.eveonline.com/Render/10000001_32.png',
                64: 'https://image.eveonline.com/Render/10000001_64.png',
                128: 'https://image.eveonline.com/Render/10000001_128.png',
                256: 'https://image.eveonline.com/Render/10000001_256.png',
                512: 'https://image.eveonline.com/Render/10000001_512.png',
            },
        })

    def test_convert_choices_to_dict(self):
        """ Test that convert_choices_to_dict properly converts. """

        choices = [
            (10000004L, 'UUA-F4'),
            (10000017L, 'J7HZ-F'),
            (10000001L, 'Derelik'),
        ]

        test_question     = TestableBaseQuestion(self.mock_db_connection)
        converted_choices = test_question.convert_choices_to_dict(choices)

        self.assertEqual(converted_choices, [
            {'text': 'UUA-F4',  'value': 10000004L},
            {'text': 'J7HZ-F',  'value': 10000017L},
            {'text': 'Derelik', 'value': 10000001L},
        ])


class TestableBaseQuestion(Question):
    """ A subclass of the base class to test against. """

    def ask(self):
        pass




# The Geography category and it's sub-categories
geography = {
    'name': 'Geography',
    'description': 'Questions regarding the solar systems and regions of New Eden.',
}

# Seoncdary Cateogries
geography_regions = {
    'name': 'Regions',
    'description': 'Questions specifically about the regions of New Eden.',
}

geography_miscellaneous = {
    'name': 'Miscellaneous',
    'description': 'Miscellaneous questions about the geography of New Eden.',
}

# The Inventory category it is's sub-categories
inventory = {
    'name': 'Inventory',
    'description': 'Questions regarding the ships, modules, and other items.',
}

inventory_slots = {
    'name': 'Slots',
    'description': 'Questions about the number of different slots ships have.',
}

inventory_ship_id = {
    'name': 'Ship ID',
    'description': 'Questions asking to identify what ship is being described.',
}




import logging

from thanatos.database.db_utils import execute_sql

_log = logging.getLogger('thanatos.database.universe')


def get_all_regions(db_connection):
    """ Gets a list of all regions.

    :return: A list of all regions. Results have regionID and regionName.
    :rtype: list
    """

    sql = 'CALL get_all_regions();'
    results = execute_sql(sql, db_connection)

    return results


def get_all_not_wh_regions(db_connection):
    """ Gets a list of all regions that are not WH regions.

    :return: A list of all regions not including wormhole regions. Results have regionID and regionName.
    :rtype: list
    """

    sql = 'CALL get_all_not_wh_regions();'
    results = execute_sql(sql, db_connection)

    return results


def get_all_regions_connected_to_region(db_connection, region_id):
    """ Gets a list of all regions connected to the region ID passed in.

    :param region_id: Region ID to find all regions connected to.
    :type region_id: int

    :return: A list of all regions connected to the specified region ID. Results have regionID and regionName.
    :rtype: list
    """

    sql = 'CALL get_all_regions_connected_to_region({});'.format(region_id)
    results = execute_sql(sql, db_connection)

    return results




import os
import bz2
import urllib
import MySQLdb
import logging

_log = logging.getLogger('thanatos.database.db_utils')

# Base URL for where to download SQL tables from
# Using Fuzztsteve's site for now
_base_url = 'https://www.fuzzwork.co.uk/dump/latest/'

# File extension to be used in the download, save, and loading
_table_file_extension = '.sql.bz2'

# A list of all tables required by the different procs.
# If adding a new proc make sure it's required tables are listed here.
_required_tables = [
    'dgmTypeAttributes',
    'invTypes',
    'invGroups',
    'invCategories',
    'mapRegions',
    'mapRegionJumps',
]


def execute_sql(sql, db_connection, fetch='all'):
    """

    :param sql:
    :param db_connection:
    :param fetch: A string of either all or one.

    :return:
    """

    cursor = db_connection.cursor()
    cursor.execute(sql)

    if fetch == 'all':
        results = [x for x in cursor.fetchall()]

    elif fetch == 'one':
        results = cursor.fetchone()

    else:
        results = None

    cursor.close()

    return results


def update_sql_stored_procs(db_connection):
    """

    :param db_connection:

    :return:
    """

    sql_dir_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'sql')

    for sql_file_name in os.listdir(sql_dir_path):
        _log.info('Running {}'.format(sql_file_name))

        sql_file_path = os.path.join(sql_dir_path, sql_file_name)
        sql_file = open(sql_file_path)
        sql = " ".join(sql_file.readlines())

        execute_sql(sql, db_connection, fetch=None)


def get_connection(connection_details=None):
    """ Creates a connection to the MySQL DB. """

    if connection_details is None:
        connection_details = get_default_connection_details()

    return MySQLdb.connect(
        connection_details['host'],
        connection_details['user'],
        connection_details['password'],
        connection_details['database']
    )


def get_default_connection_details():
    """ Gets the connection details based on environment vars or Thanatos default settings.

    :return: Returns a dictionary of connection details.
    :rtype: dict
    """

    _log.info('Getting default database connection details.')

    if os.environ.get('C9_PROJECT') is not None:
        _log.info('C9_PROJECT environment found. Getting C9 DB connection details.')

        # See https://docs.c9.io/v1.0/docs/setting-up-mysql

        return {
            'host': os.environ.get('IP'),
            'user': os.environ.get('C9_USER'),
            'password': '',
            'database': 'c9',
        }

    else:
        _log.info('Getting Vagrant DB default connection details.')

        return {
            'host': '127.0.0.1',
            'user': 'vagrant',
            'password': 'vagrant',
            'database': 'thanatos',
        }


def load_tables_from_files(db_connection, tables=_required_tables):
    """ Looks in the current working directory for all required tables. """

    _log.info('Loading tables from disk to DB.')

    for table in tables:
        _log.info('Loading the following table: {}'.format(table))

        table_name = table + _table_file_extension

        with bz2.BZ2File(table_name, 'r') as sql_file:
            sql = sql_file.read()
            execute_sql(sql, db_connection)

    _log.info('Finished loading all requested tables.')


def download_tables(tables_list=_required_tables, base_url=_base_url):
    """ Downloads a given list of tables from a given web site.

    :param tables_list: A list of strings, each a table name to download.
    :type tables_list: list

    :return:
    """

    _log.info('Downloading all required tables.')

    formatting_string = base_url + '{}' + _table_file_extension

    for table in tables_list:
        full_url = formatting_string.format(table)

        _log.info('Downloading {}'.format(full_url))

        urllib.urlretrieve(full_url, table + _table_file_extension)

    _log.info('Finished downloading all required tables.')